/*
 * jEdit - Programmer's Text Editor
 * :tabSize=8:indentSize=8:noTabs=false:
 * :folding=explicit:collapseFolds=1:
 *
 * Copyright © 2020 jEdit contributors
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.gjt.sp.util.swing.event;

import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.util.function.Consumer;

/**
 * A simple document event that does the same action for insertUpdate, removeUpdate and changeUpdate
 * @author Matthieu Casanova
 */
public class UniqueActionDocumentListener implements DocumentListener
{
	private final Consumer<DocumentEvent> consumer;

	public UniqueActionDocumentListener(Consumer<DocumentEvent> consumer)
	{
		this.consumer = consumer;
	}

	@Override
	public void insertUpdate(DocumentEvent e)
	{
		consumer.accept(e);
	}

	@Override
	public void removeUpdate(DocumentEvent e)
	{
		consumer.accept(e);
	}

	@Override
	public void changedUpdate(DocumentEvent e)
	{
		consumer.accept(e);
	}
}
